/**************************************************************************

	nLaunch

	image.c

	Copyright (C) 1996-2002 by Tomoaki Nakashima. All rights reserved.
		http://www.nakka.com/
		nakka@nakka.com

**************************************************************************/

/**************************************************************************
	Include Files
**************************************************************************/

#define _INC_OLE
#include <windows.h>
#undef  _INC_OLE
#include <commctrl.h>

#include "general.h"
#include "image.h"

#include "resource.h"


/**************************************************************************
	Define
**************************************************************************/

#define BUF_SIZE				256							/* obt@TCY */


/**************************************************************************
	Global Variables
**************************************************************************/

static HIMAGELIST IconList;			/* C[WXg */
static int IconListCnt;

extern HINSTANCE hInst;
extern int TransparentIcon;


/**************************************************************************
	Local Function Prototypes
**************************************************************************/

static void ImageListIconAdd(HIMAGELIST IconList, int IconSize, char *path, char *File);
static HFONT CreateTitleFont(HDC hdc, PAINTINFO *pi);
static int GetFontSize(PAINTINFO *pi);
static BOOL DrawBackground(HDC hdc, char *fbmp, long color, int width, int height, int mode);
static void DrawIconFocus(HDC hdc, PAINTINFO *pi);


/******************************************************************************

	ImageListIconAdd

	C[WXgɃACRǉ

******************************************************************************/

static void ImageListIconAdd(HIMAGELIST IconList, int IconSize, char *path, char *File)
{
	HICON hIcon = NULL;
	SHFILEINFO  shfi;
	char *buf = NULL;

	if(path != NULL && File != NULL){
		buf = (char *)LocalAlloc(LMEM_FIXED, lstrlen(path) + lstrlen(File) + 2);
	}
	if(buf != NULL){
		wsprintf(buf, "%s\\%s", path, File);
		/* ֘AtꂽACR̎擾 */
		SHGetFileInfo(buf, SHGFI_USEFILEATTRIBUTES, &shfi, sizeof(SHFILEINFO),
			SHGFI_ICON | ((IconSize < 24) ? SHGFI_SMALLICON : 0));
		hIcon = shfi.hIcon;
		LocalFree(buf);
	}
	if(hIcon == NULL){
		/* AvP[VACR̎擾 */
		hIcon = LoadIcon(0, IDI_APPLICATION);
	}

	/* C[WXgɃACRǉ */
	ImageList_AddIcon(IconList, hIcon);
	DestroyIcon(hIcon);
}


/******************************************************************************

	SetImegeListIcon

	C[WXgݒ

******************************************************************************/

void SetImegeListIcon(char *path, FILELIST *FileList, int IconCnt, int IconSize, PAINTINFO *pi)
{
	OSVERSIONINFO os_info;
	HICON hIcon;
	UINT color_flag = ILC_COLOR16;
	int i;

	/* FtO̐ݒ */
	os_info.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	GetVersionEx(&os_info);
	if((os_info.dwPlatformId == VER_PLATFORM_WIN32_NT) &&
		(os_info.dwMajorVersion > 5 || (os_info.dwMajorVersion == 5 && os_info.dwMinorVersion > 0))){
		color_flag = ILC_COLOR32;
	}

	/* C[WXg̍쐬 */
	if(IconList != NULL){
		ImageList_Destroy((void *)IconList);
	}
	IconListCnt = IconCnt;
	IconList = ImageList_Create(IconSize, IconSize, color_flag | ILC_MASK, IconListCnt, IconListCnt);
	ImageList_SetBkColor(IconList,
		(pi->WindowsDef == 1) ? GetSysColor(COLOR_WINDOW) : pi->FocusBackColor);

	/* LZACR̒ǉ */
	hIcon = LoadIcon(hInst, MAKEINTRESOURCE(IDI_ICON_NON));
	ImageList_AddIcon(IconList, hIcon);
	DestroyIcon(hIcon);

	/* t@CACR̒ǉ */
	for(i = 1; i < IconListCnt; i++){
		if(FileList != NULL){
			ImageListIconAdd(IconList, IconSize, path, (FileList + i)->FileName);
			(FileList + i)->ImageIndex = i;
		}else{
			ImageListIconAdd(IconList, IconSize, path, NULL);
		}
	}
}


/******************************************************************************

	EndImegeListIcon

	C[WXg̔j

******************************************************************************/

void EndImegeListIcon(void)
{
	/* C[WXg̔j */
	if(IconList != NULL){
		ImageList_Destroy((void *)IconList);
		IconList = NULL;
	}
}


/******************************************************************************

	CreateTitleFont

	^Cgo[̃tHg쐬

******************************************************************************/

static HFONT CreateTitleFont(HDC hdc, PAINTINFO *pi)
{
	LOGFONT LogFont;

	ZeroMemory(&LogFont, sizeof(LOGFONT));
	LogFont.lfHeight = -(int)((pi->FontHeight * GetDeviceCaps(hdc, LOGPIXELSY)) / 72);
	LogFont.lfWeight = pi->FontWeight;
	LogFont.lfItalic = pi->FontItalic;
	LogFont.lfCharSet = DEFAULT_CHARSET;
	lstrcpy(LogFont.lfFaceName, pi->FontName);
	/* tHg̍쐬 */
	return CreateFontIndirect(&LogFont);
}


/******************************************************************************

	GetFontSize

	tHg̍擾

******************************************************************************/

static int GetFontSize(PAINTINFO *pi)
{
	HDC hdc;
	HFONT hRetFont, hFont = NULL;
	TEXTMETRIC lptm;

	hdc = GetDC(NULL);
	if(pi->WindowsDef == 0){
		/* tHg̍쐬 */
		hFont = CreateTitleFont(hdc, pi);
		hRetFont = SelectObject(hdc, hFont);
	}else{
		/* ftHgtHg̐ݒ */
		hRetFont = SelectObject(hdc, GetStockObject(DEFAULT_GUI_FONT));
	}
	GetTextMetrics(hdc, &lptm);
	SelectObject(hdc, hRetFont);
	ReleaseDC(NULL, hdc);
	if(hFont != NULL) DeleteObject(hFont);
	return lptm.tmHeight + lptm.tmExternalLeading;
}


/******************************************************************************

	GetTitleSize

	^Cgo[̍擾

******************************************************************************/

int GetTitleSize(PAINTINFO *pi)
{
	int ret;

	ret = GetFontSize(pi);
	return ((pi->TitleSize > ret) ? pi->TitleSize : ret);
}


/******************************************************************************

	DrawBackground

	wi̕`

******************************************************************************/

static BOOL DrawBackground(HDC hdc, char *fbmp, long color, int width, int height, int mode)
{
	HDC tmpdc;
	HBITMAP hBmp = NULL, hrBmp;
	HBRUSH hbrush;
	BITMAP bmp;
	RECT rect;
	int i, j;

	if(color != -1){
		/* wFœhԂ */
		rect.left = rect.top = 0;
		rect.right = width;
		rect.bottom = height;
		hbrush = CreateSolidBrush(color);
		FillRect(hdc, &rect, hbrush);
		DeleteObject(hbrush);
	}

	if(fbmp != NULL && *fbmp != '\0'){
		/* 摜̓ǂݍ */
		hBmp = LoadImage(NULL, fbmp, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);
	}
	if(hBmp != NULL){
		tmpdc = CreateCompatibleDC(hdc);
		hrBmp = SelectObject(tmpdc, hBmp);
		/* ^C\ */
		GetObject(hBmp, sizeof(bmp), &bmp);
		switch(mode)
		{
		case 0:
			/*  */
			BitBlt(hdc,
				width / 2 - bmp.bmWidth / 2, height / 2 - bmp.bmHeight / 2,
				bmp.bmWidth, bmp.bmHeight, tmpdc, 0, 0, SRCCOPY);
			break;

		case 1:
			/* ^C */
			for(i = 0; width > (bmp.bmWidth * i); i++){
				for(j = 0; height > (bmp.bmHeight * j); j++){
					BitBlt(hdc,
						bmp.bmWidth * i, bmp.bmHeight * j, bmp.bmWidth, bmp.bmHeight,
						tmpdc, 0, 0, SRCCOPY);
				}
			}
			break;

		case 2:
			/* g */
			StretchBlt(hdc, 0, 0, width, height,
				tmpdc, 0, 0, bmp.bmWidth, bmp.bmHeight, SRCCOPY);
			break;
		}
		SelectObject(tmpdc, hrBmp);
		DeleteObject(hBmp);
		DeleteDC(tmpdc);
		return TRUE;
	}
	return FALSE;
}


/******************************************************************************

	_TransparentBlt

	ߐFgDCRs[

******************************************************************************/

void _TransparentBlt(HDC todc, int left, int top, int width, int height,
					HDC fromdc, int fleft, int ftop, COLORREF color)
{
	HDC maskDC;
	HBITMAP hMaskBmp;
	HBITMAP hRetMaskBmp;
	HDC tempDC;
	HBITMAP hBmp;
	HBITMAP hRetBmp;

	/* Oi̍쐬 */
	tempDC = CreateCompatibleDC(todc);
	hBmp = CreateCompatibleBitmap(todc, width, height);
	hRetBmp = SelectObject(tempDC, hBmp);
	BitBlt(tempDC, 0, 0, width, height, fromdc, fleft, ftop, SRCCOPY);

	/* }XN̍쐬 */
	maskDC = CreateCompatibleDC(todc);
	hMaskBmp = CreateBitmap(width, height, 1, 1, NULL);
	hRetMaskBmp = SelectObject(maskDC, hMaskBmp);
	SetBkColor(tempDC, color);
	BitBlt(maskDC, 0, 0, width, height, tempDC, 0, 0, SRCCOPY);

	/* }XNgđOiwǐ` */
	SetBkColor(tempDC, RGB(0, 0, 0));
	SetTextColor(tempDC, RGB(255, 255, 255));
	BitBlt(tempDC, 0, 0, width, height, maskDC, 0, 0, SRCAND);

	/* }XNgĔwiOǐ` */
	SetBkColor(todc, RGB(255, 255, 255));
	SetTextColor(todc, RGB(0, 0, 0));
	BitBlt(todc, left, top, width, height, maskDC, 0, 0, SRCAND);

	/* OiƔwi */
	BitBlt(todc, left, top, width, height, tempDC, 0, 0, SRCPAINT);

	SelectObject(maskDC, hRetMaskBmp);
	DeleteObject(hMaskBmp);
	DeleteDC(maskDC);

	SelectObject(tempDC, hRetBmp);
	DeleteObject(hBmp);
	DeleteDC(tempDC);
}


/******************************************************************************

	DrawIconFocus

	tH[JX̕`

******************************************************************************/

static void DrawIconFocus(HDC hdc, PAINTINFO *pi)
{
	HDC tmpdc;
	HBITMAP hBmp, hrBmp;
	HBRUSH hbrush;
	RECT rect;
	long color;
	int left, top;

	/* tH[JẌʒu̐ݒ */
	left = pi->WindowWidth / 2 - pi->FocusSize / 2;
	top = pi->IconTop + (pi->IconSize / 2) - pi->FocusSize / 2;

	/* tH[JXrbg}bv̍쐬 */
	tmpdc = CreateCompatibleDC(hdc);
	hBmp = CreateCompatibleBitmap(hdc, pi->FocusSize, pi->FocusSize);
	hrBmp = SelectObject(tmpdc, hBmp);
	color = (pi->WindowsDef == 1) ? GetSysColor(COLOR_WINDOW) : pi->FocusBackColor;
	DrawBackground(tmpdc, pi->FocusBitmap, color, pi->FocusSize, pi->FocusSize, pi->FocusPaintMode);

	/*  */
	if(pi->TransparentWindow == 1){
		HDC bkdc;
		HBITMAP hBkBmp, hrBkBmp;

		/* ߗpDC̍쐬 */
		bkdc = CreateCompatibleDC(hdc);
		hBkBmp = CreateCompatibleBitmap(hdc, pi->FocusSize, pi->FocusSize);
		hrBkBmp = SelectObject(bkdc, hBkBmp);
		BitBlt(bkdc, 0, 0, pi->FocusSize, pi->FocusSize, hdc, left, top, SRCCOPY);

		/* ߂ĕ` */
		_TransparentBlt(bkdc, 0, 0, pi->FocusSize, pi->FocusSize,
			tmpdc, 0, 0, pi->TransparentColor);
		/* DCɖ߂ */
		BitBlt(tmpdc, 0, 0, pi->FocusSize, pi->FocusSize, bkdc, 0, 0, SRCCOPY);

		SelectObject(bkdc, hrBkBmp);
		DeleteObject(hBkBmp);
		DeleteDC(bkdc);
	}

	/* tH[JX̘g̕` */
	rect.left = left - pi->FocusFrameSize;
	rect.top = top - pi->FocusFrameSize;
	rect.right = rect.left + pi->FocusSize + pi->FocusFrameSize * 2;
	rect.bottom = rect.top + pi->FocusSize + pi->FocusFrameSize * 2;
	hbrush = CreateSolidBrush(((pi->WindowsDef == 1) ?
		GetSysColor(COLOR_WINDOWFRAME) : pi->FocusFrameColor));
	FillRect(hdc, &rect, hbrush);
	DeleteObject(hbrush);

	/* tH[JXrbg}bv̕` */
	BitBlt(hdc, left, top, pi->FocusSize, pi->FocusSize, tmpdc, 0, 0, SRCCOPY);
	SelectObject(tmpdc, hrBmp);
	DeleteObject(hBmp);
	DeleteDC(tmpdc);
}


/******************************************************************************

	InitPanel

	`pDC쐬

******************************************************************************/

void InitPanel(PAINTINFO *pi)
{
	HDC hdc;
	HBRUSH hbrush;
	RECT rect;
	int ret;

	hdc = GetDC(NULL);

	/* ACRƍF̐ݒ */
	if(TransparentIcon == 0 || GetDeviceCaps(hdc, BITSPIXEL) <= 8){
		/* wiFƍ */
		pi->TransColor = (pi->WindowsDef == 1) ?
			GetSysColor(COLOR_3DFACE) : pi->WindowColor;
	}else{
		/* wi摜ƍ */
		pi->TransColor = CLR_NONE;
	}

	/* ^Cgo[̍擾 */
	pi->TitleHeight = GetTitleSize(pi);
	/* ^CgeLXg̈ʒu̐ݒ */
	ret = GetFontSize(pi);
	if(pi->TitleTextTop + ret > pi->TitleHeight){
		pi->TitleTextTop = pi->TitleHeight - ret;
	}

	/* plTCY̐ݒ */
	if(pi->IconWidth < pi->IconSize){
		pi->IconWidth = pi->IconSize;
	}
	pi->WindowWidth = pi->IconWidth * pi->IconCount + pi->IconLeft * 2;
	if(pi->WindowHeight < pi->IconSize + pi->IconTop){
		pi->WindowHeight = pi->IconSize + pi->IconTop;
	}

	/* plrbg}bv̍쐬 */
	rect.left = rect.top = 0;
	rect.right = pi->WindowWidth + pi->FrameSize * 2;
	rect.bottom = pi->WindowHeight + pi->TitleHeight * pi->ShowTitle + pi->FrameSize * 2;

	pi->hPanelDC = CreateCompatibleDC(hdc);
	pi->hPanelBmp = CreateCompatibleBitmap(hdc, rect.right, rect.bottom);
	pi->hRetPanelBmp = SelectObject(pi->hPanelDC, pi->hPanelBmp);
	/* plt[FœhԂ */
	hbrush = CreateSolidBrush(((pi->WindowsDef == 1) ?
		((pi->FlatWindow == 1) ?
		GetSysColor(COLOR_WINDOWFRAME) : GetSysColor(COLOR_ACTIVEBORDER))
		: pi->FrameColor));
	FillRect(pi->hPanelDC, &rect, hbrush);
	DeleteObject(hbrush);
	/* tHg̐ݒ */
	if(pi->WindowsDef == 0){
		pi->hFont = CreateTitleFont(pi->hPanelDC, pi);
		pi->hRetFont = SelectObject(pi->hPanelDC, pi->hFont);
		SetTextColor(pi->hPanelDC, pi->TitleTextColor);
	}else{
		pi->hFont = NULL;
		pi->hRetFont = SelectObject(pi->hPanelDC, GetStockObject(DEFAULT_GUI_FONT));
		SetTextColor(pi->hPanelDC, GetSysColor(COLOR_CAPTIONTEXT));
	}
	SetBkMode(pi->hPanelDC, TRANSPARENT);

	/* wirbg}bv̍쐬 */
	pi->hBackDC = CreateCompatibleDC(hdc);
	pi->hBackBmp = CreateCompatibleBitmap(hdc, pi->WindowWidth, pi->WindowHeight);
	pi->hRetBackBmp = SelectObject(pi->hBackDC, pi->hBackBmp);
	/* wi摜̓ǂݍ */
	DrawBackground(pi->hBackDC, pi->Background,
		((pi->WindowsDef == 1) ? GetSysColor(COLOR_3DFACE) : pi->WindowColor),
		pi->WindowWidth, pi->WindowHeight, pi->BackPaintMode);
	/* wiɃtH[JX` */
	DrawIconFocus(pi->hBackDC, pi);
	/* wiplɕ` */
	BitBlt(pi->hPanelDC, pi->FrameSize,
		pi->FrameSize + ((pi->DownTitle == 1) ? 0 : (pi->TitleHeight * pi->ShowTitle)),
		pi->WindowWidth, pi->WindowHeight,
		pi->hBackDC, 0, 0, SRCCOPY);

	/* ^Cgrbg}bv̍쐬 */
	pi->hTitleDC = CreateCompatibleDC(hdc);
	pi->hTitleBmp = CreateCompatibleBitmap(hdc, pi->WindowWidth, pi->TitleHeight);
	pi->hRetTitleBmp = SelectObject(pi->hTitleDC, pi->hTitleBmp);
	/* wi摜̓ǂݍ */
	if(DrawBackground(pi->hTitleDC, pi->Titlebar,
		((pi->WindowsDef == 1) ? GetSysColor(COLOR_ACTIVECAPTION) : pi->TitleColor),
		pi->WindowWidth, pi->TitleHeight, pi->TitlePaintMode) == FALSE && pi->WindowsDef == 1){
		/* Windows̃^Cgo[gp邽ߍ쐬rbg}bvj */
		SelectObject(pi->hTitleDC, pi->hRetTitleBmp);
		DeleteObject(pi->hTitleBmp);
		DeleteDC(pi->hTitleDC);
		pi->hTitleDC = NULL;
	}
	ReleaseDC(NULL, hdc);
}


/******************************************************************************

	EndPanel

	`pDCj

******************************************************************************/

void EndPanel(PAINTINFO *pi)
{
	/* tHg̔j */
	SelectObject(pi->hPanelDC, pi->hRetFont);
	if(pi->hFont != NULL) DeleteObject(pi->hFont);

	/* pl̔j */
	SelectObject(pi->hPanelDC, pi->hRetPanelBmp);
	DeleteObject(pi->hPanelBmp);
	DeleteDC(pi->hPanelDC);

	/* wi̔j */
	SelectObject(pi->hBackDC, pi->hRetBackBmp);
	DeleteObject(pi->hBackBmp);
	DeleteDC(pi->hBackDC);

	/* ^Cg̔j */
	if(pi->hTitleDC != NULL){
		SelectObject(pi->hTitleDC, pi->hRetTitleBmp);
		DeleteObject(pi->hTitleBmp);
		DeleteDC(pi->hTitleDC);
		pi->hTitleDC = NULL;
	}
}


/******************************************************************************

	SetIconPanel

	\pl̐ݒ

******************************************************************************/

void SetIconPanel(HWND hWnd, PAINTINFO *pi, FILELIST *FileList, int ViewIndex, int ViewPos)
{
#ifndef DC_GRADIENT
#define DC_GRADIENT		32
#endif
	HDC hIconDC;
	HBITMAP hIconBmp, hRetIconBmp;
	RECT rect;
	int IconIndex, i;
	int left, top;
	int width;

	if(ViewIndex == -1 || IconListCnt == 0){
		/* wî݂̕` */
		BitBlt(pi->hPanelDC, pi->FrameSize,
			pi->FrameSize + ((pi->DownTitle == 1) ? 0 : (pi->TitleHeight * pi->ShowTitle)),
			pi->WindowWidth, pi->WindowHeight,
			pi->hBackDC, 0, 0, SRCCOPY);
	}else{
		/* wirbg}bv̍쐬 */
		left = pi->WindowWidth / 2 - (pi->IconWidth * pi->IconCount) / 2;
		top = pi->IconTop;
		hIconDC = CreateCompatibleDC(pi->hPanelDC);
		hIconBmp = CreateCompatibleBitmap(pi->hPanelDC, pi->IconWidth * pi->IconCount, pi->IconSize);
		hRetIconBmp = SelectObject(hIconDC, hIconBmp);
		BitBlt(hIconDC, 0, 0, pi->IconWidth * pi->IconCount, pi->IconSize, pi->hBackDC, left, top, SRCCOPY);

		/* ACR̕` */
		IconIndex = ViewIndex - ((pi->IconCount + 2) / 2);
		if(IconIndex < 0){
			IconIndex %= IconListCnt;
			IconIndex = IconListCnt + IconIndex;
		}
		width = (pi->IconWidth - pi->IconSize) / 2 + ViewPos -
			((pi->IconCount / 2 * 2 == pi->IconCount) ? (pi->IconWidth / 2) : 0);
		for(i = -1; i <= pi->IconCount + 1; i++){
			ImageList_DrawEx(IconList, ((FileList != NULL) ? (FileList + IconIndex)->ImageIndex : IconIndex),
				hIconDC, pi->IconWidth * i + width, 0, pi->IconSize, pi->IconSize, CLR_NONE, pi->TransColor,
				ILD_TRANSPARENT | ((i == pi->IconCount / 2) ? ILD_NORMAL : ILD_BLEND50));
			IconIndex++;
			if(IconIndex >= IconListCnt){
				IconIndex = IconIndex - IconListCnt;
			}
		}
		/* plɕ` */
		left += pi->FrameSize;
		top += pi->FrameSize + ((pi->DownTitle == 1) ? 0 : (pi->TitleHeight * pi->ShowTitle));
		BitBlt(pi->hPanelDC, left, top, pi->IconWidth * pi->IconCount, pi->IconSize, hIconDC, 0, 0, SRCCOPY);
		SelectObject(hIconDC, hRetIconBmp);
		DeleteObject(hIconBmp);
		DeleteDC(hIconDC);
	}

	if(pi->ShowTitle == 0){
		/* ^Cg\Ȃ */
		return;
	}

	/* ^Cgo[̕` */
	rect.left = pi->FrameSize;
	rect.top = pi->FrameSize + ((pi->DownTitle == 1) ? pi->WindowHeight : 0);
	rect.right = rect.left + pi->WindowWidth;
	rect.bottom = rect.top + pi->TitleHeight;
	if(pi->WindowsDef == 1 && pi->hTitleDC == NULL){
		/* Windows̃ftHg̃^Cgo[` */
		DrawCaption(hWnd, pi->hPanelDC, &rect, DC_ACTIVE | DC_SMALLCAP | DC_TEXT | DC_GRADIENT);
	}else{
		char buf[BUF_SIZE];

		/* 摜^Cgo[ɕ` */
		BitBlt(pi->hPanelDC, rect.left, rect.top,
			rect.right, rect.bottom, pi->hTitleDC, 0, 0, SRCCOPY);

		/* eLXg̕` */
		rect.left += pi->TitleTextLeft;
		rect.right -= pi->TitleTextRight;
		if(pi->TitleTextTop != 0){
			rect.top += pi->TitleTextTop;
		}
		GetWindowText(hWnd, buf, BUF_SIZE - 1);
		DrawText(pi->hPanelDC, buf, lstrlen(buf), &rect,
			DT_SINGLELINE | DT_MODIFYSTRING | DT_END_ELLIPSIS | DT_NOPREFIX |
			((pi->TitleTextTop == 0) ? DT_VCENTER : 0) |
			((pi->CenterText == 1) ? DT_CENTER : 0));
	}
}


/******************************************************************************

	PanelHitTest

	ʒuꏊ擾

******************************************************************************/

int PanelHitTest(PAINTINFO *pi, int x, int y, int ViewIndex)
{
	RECT rect;
	POINT ps;
	int ret = POS_FRAME;
	int IconIndex;
	int width;
	int i;

	ps.x = x; ps.y = y;

	/* EBhE̔wi */
	rect.left = pi->FrameSize;
	rect.top = pi->FrameSize +
		((pi->DownTitle == 1) ? 0 : (pi->TitleHeight * pi->ShowTitle));
	rect.right = rect.left + pi->WindowWidth;
	rect.bottom = rect.top + pi->WindowHeight;
	if(PtInRect(&rect, ps) == TRUE){
		ret = POS_WINDOW;
	}

	/* ACR */
	if(IconListCnt > 0){
		IconIndex = ViewIndex - ((pi->IconCount + 2) / 2);
		if(IconIndex < 0){
			IconIndex %= IconListCnt;
			IconIndex = IconListCnt + IconIndex;
		}
		width = (pi->IconWidth - pi->IconSize) / 2 -
			((pi->IconCount / 2 * 2 == pi->IconCount) ? (pi->IconWidth / 2) : 0) + pi->FrameSize + pi->IconLeft;
		rect.top = pi->IconTop + pi->FrameSize + ((pi->DownTitle == 1) ? 0 : (pi->TitleHeight * pi->ShowTitle));
		rect.bottom = rect.top + pi->IconSize;
		for(i = -1; i <= pi->IconCount + 1; i++){
			rect.left = pi->IconWidth * i + width;
			rect.right = rect.left + pi->IconSize;

			if(PtInRect(&rect, ps) == TRUE){
				ret = POS_ICONINDEX + IconIndex;
			}

			IconIndex++;
			if(IconIndex >= IconListCnt){
				IconIndex = IconIndex - IconListCnt;
			}
		}
	}

	/* tH[JXg */
	rect.left = (pi->WindowWidth) / 2 -
		pi->FocusSize / 2 - pi->FocusFrameSize + pi->FrameSize;
	rect.top = pi->IconTop + (pi->IconSize / 2) - pi->FocusSize / 2 -
		pi->FocusFrameSize + pi->FrameSize +
		((pi->DownTitle == 1) ? 0 : (pi->TitleHeight * pi->ShowTitle));
	rect.right = rect.left + pi->FocusSize + pi->FocusFrameSize * 2;
	rect.bottom = rect.top + pi->FocusSize + pi->FocusFrameSize * 2;
	if(PtInRect(&rect, ps) == TRUE){
		ret = POS_SELICONFRAME;
	}

	/* tH[JX̔wi */
	rect.left += pi->FocusFrameSize;
	rect.top += pi->FocusFrameSize;
	rect.right -= pi->FocusFrameSize;
	rect.bottom -= pi->FocusFrameSize;
	if(PtInRect(&rect, ps) == TRUE){
		ret = POS_SELICON;
	}

	/* ^Cgo[ */
	rect.left = pi->FrameSize;
	rect.top = pi->FrameSize + ((pi->DownTitle == 1) ? pi->WindowHeight : 0);
	rect.right = rect.left + pi->WindowWidth;
	rect.bottom = rect.top + pi->TitleHeight;
	if(pi->ShowTitle == 1 && PtInRect(&rect, ps) == TRUE){
		ret = POS_TITLEBAR;
		/* ^Cgo[̃eLXg */
		if(pi->CenterText == 0){
			rect.right = rect.left + pi->FontHeight * 4 + pi->TitleTextLeft;
		}else{
			rect.left = pi->FrameSize + pi->TitleTextLeft +
				(pi->WindowWidth - (pi->TitleTextRight + pi->TitleTextLeft)) / 2 -
				(pi->FontHeight * 4) / 2;
			rect.right = rect.left + pi->FontHeight * 4;
		}
		if(PtInRect(&rect, ps) == TRUE){
			ret = POS_TITLETEXT;
		}
	}
	return ret;
}


/******************************************************************************

	CreateBmpRgn

	rbg}bv̓F[W쐬

******************************************************************************/

HRGN CreateBmpRgn(HBITMAP hbmp, COLORREF color, int FrameWidth, int FrameHeight)
{
#define COLOR_BITS		24	/* 24bit color */
	HDC hdc;
	HRGN hRgn;
    HRGN hTmpRgn;
	BITMAPINFO bi;
	BITMAP bmp;
	LPBYTE bits, p;
	DWORD LineSize;
	int x, y, st;

	/* rbg}bv̏擾 */
	if(hbmp == NULL || GetObject(hbmp, sizeof(bmp), &bmp) == 0){
		return NULL;
	}

	/* 1C̃TCYvZ (sizeof(DWORD) ̔{ɂ) */
	LineSize = (bmp.bmWidth * (COLOR_BITS / 8) + 3) & ~3;

	/* wb_̏ */
	ZeroMemory(&bi, sizeof(BITMAPINFOHEADER));
	bi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	bi.bmiHeader.biWidth = bmp.bmWidth;
	bi.bmiHeader.biHeight = bmp.bmHeight;
	bi.bmiHeader.biPlanes = 1;
	bi.bmiHeader.biBitCount = COLOR_BITS;
	bi.bmiHeader.biCompression = BI_RGB;
	bi.bmiHeader.biSizeImage = LineSize * bmp.bmHeight;

	/* DIB̃rbgi[郁m */
	bits = LocalAlloc(LMEM_FIXED, LineSize * bmp.bmHeight);
	if(bits == NULL){
		return NULL;
	}

	hdc = GetDC(NULL);
	/* rbg擾 */
	if(GetDIBits(hdc, hbmp, 0, bmp.bmHeight, (LPVOID)bits, &bi, DIB_RGB_COLORS) == 0){
		ReleaseDC(NULL, hdc);
		LocalFree(bits);
		return NULL;
	}
	ReleaseDC(NULL, hdc);

	/* EBhETCỸ[W̍쐬 */
	hRgn = CreateRectRgn(0, 0, bmp.bmWidth + FrameWidth * 2, bmp.bmHeight + FrameHeight * 2);

	/* rbg}bv̓F[W珜 */
	for(y = 0; y < bmp.bmHeight; y++){
		x = 0;
		p = bits + LineSize * y;
		/* 1C */
		while(x < bmp.bmWidth){
			/* F̎n_ƏI_擾 */
			for(st = x; x < bmp.bmWidth &&
				(COLORREF)(*p << 16 | *(p + 1) << 8 | *(p + 2)) == color; x++, p += (COLOR_BITS / 8));
			if(st == x){
				/* Fł͂Ȃꍇ */
				x++;
				p += (COLOR_BITS / 8);
				continue;
			}
			/* [W瓧F̕ */
			hTmpRgn = CreateRectRgn(st + FrameWidth, bmp.bmHeight - y + FrameHeight,
				x + FrameWidth, bmp.bmHeight - y - 1 + FrameHeight);
			CombineRgn(hRgn, hRgn, hTmpRgn, RGN_XOR);
			DeleteObject(hTmpRgn);
		}
	}
	LocalFree(bits);
    return hRgn;
}
/* End of source */
